/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.entity;

import org.apache.ranger.common.AppConstants;

import javax.persistence.*;
import javax.xml.bind.annotation.XmlRootElement;
import java.io.Serializable;
import java.util.Objects;

@Entity
@Cacheable
@Table(name="x_gds_project")
@XmlRootElement
public class XXGdsProject extends XXDBBase implements Serializable {
    private static final long serialVersionUID = 1L;

    @Id
    @SequenceGenerator(name = "X_GDS_PROJECT_SEQ", sequenceName = "X_GDS_PROJECT_SEQ", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "X_GDS_PROJECT_SEQ")
    @Column(name = "id")
    protected Long id;

    @Column(name = "guid", unique = true, nullable = false, length = 512)
    protected String guid;

    @Version
    @Column(name = "version")
    protected Long version;

    @Column(name = "is_enabled")
    protected Boolean isEnabled;

    @Column(name = "name")
    protected String name;

    @Column(name = "description")
    protected String description;

    @Column(name = "acl")
    protected String acl;

    @Column(name = "terms_of_use")
    protected String termsOfUse;

    @Column(name = "options")
    protected String options;

    @Column(name = "additional_info")
    protected String additionalInfo;

    @Override
    public void setId(Long id) { this.id = id; }

    @Override
    public Long getId() { return id; }

    public String getGuid() { return guid; }

    public void setGuid(String guid) { this.guid = guid; }

    public Long getVersion() { return version; }

    public void setVersion(Long version) { this.version = version; }

    public Boolean getIsEnabled() { return isEnabled; }

    public void setIsEnabled(Boolean isEnabled) { this.isEnabled = isEnabled; }

    public String getName() { return name; }

    public void setName(String name) { this.name = name; }

    public String getDescription() { return description; }

    public void setDescription(String description) { this.description = description; }

    public String getAcl() { return acl; }

    public void setAcl(String acl) { this.acl = acl; }

    public String getTermsOfUse() { return termsOfUse; }

    public void setTermsOfUse(String termsOfUse) { this.termsOfUse = termsOfUse; }

    public String getOptions() { return options; }

    public void setOptions(String options) { this.options = options; }

    public String getAdditionalInfo() { return additionalInfo; }

    public void setAdditionalInfo(String additionalInfo) { this.additionalInfo = additionalInfo; }

    @Override
    public int getMyClassType() { return AppConstants.CLASS_TYPE_GDS_PROJECT; }

    @Override
    public int hashCode() {
        return Objects.hash(id, guid, version, isEnabled, name, description, acl, termsOfUse, options, additionalInfo);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        } else if (!super.equals(obj)) {
            return false;
        } else if (getClass() != obj.getClass()) {
            return false;
        }

        XXGdsProject other = (XXGdsProject) obj;

        return Objects.equals(id, other.id) &&
               Objects.equals(guid, other.guid) &&
               Objects.equals(version, other.version) &&
               Objects.equals(isEnabled, other.isEnabled) &&
               Objects.equals(name, other.name) &&
               Objects.equals(description, other.description) &&
               Objects.equals(acl, other.acl) &&
               Objects.equals(termsOfUse, other.termsOfUse) &&
               Objects.equals(options, other.options) &&
               Objects.equals(additionalInfo, other.additionalInfo);
    }

    @Override
    public String toString() {
        return toString(new StringBuilder()).toString();
    }

    public StringBuilder toString(StringBuilder sb) {
        sb.append("XXGdsProject={ ")
          .append(super.toString()).append(" ")
          .append("id={").append(id).append("} ")
          .append("guid={").append(guid).append("} ")
          .append("version={").append(version).append("} ")
          .append("isEnabled={").append(isEnabled).append("} ")
          .append("name={").append(name).append("} ")
          .append("description={").append(description).append("} ")
          .append("condition={").append(acl).append("} ")
          .append("termsOfUse={").append(termsOfUse).append("} ")
          .append("options={").append(options).append("} ")
          .append("additionalInfo={").append(additionalInfo).append("} ")
          .append(" }");

        return sb;
    }
}
